@@ -1,19 +1,20 @@
-from contextlib import contextmanager
 import dataclasses
+import datetime
 import os
 import pickle
+from contextlib import contextmanager
 from importlib import util
 from inspect import getsource
 from pathlib import Path
 from typing import Tuple
 
 import numpy as np
-import pandas as pd
+import pandas as pd  # type: ignore
 import pyarrow as pa  # type: ignore
 from tqdm import trange  # type: ignore
 
-from .api import ZenoOptions
-from .classes import ZenoColumn, ZenoColumnType, ZenoFunction
+from zeno.api import ZenoOptions
+from zeno.classes import MetadataType, ZenoColumn, ZenoColumnType, ZenoFunction
 
 
 @contextmanager
@@ -50,6 +51,32 @@ def read_pickle(file_name: str, cache_path: Path, default):
         return default
 
 
+def getMetadataType(col: pd.Series) -> MetadataType:
+    col = col.infer_objects()
+
+    try:
+        datetime.datetime.fromisoformat(str(col[0]))
+        return MetadataType.DATETIME
+    except ValueError:
+        pass
+
+    if col.dtype == "bool":
+        return MetadataType.BOOLEAN
+
+    try:
+        unique = col.unique().tolist()
+    except TypeError:
+        return MetadataType.OTHER
+
+    if len(unique) < 21:
+        return MetadataType.NOMINAL
+
+    if col.dtype == "int64" or col.dtype == "float64":
+        return MetadataType.CONTINUOUS
+
+    return MetadataType.OTHER
+
+
 def load_series(df, col_name, save_path):
     try:
         df.loc[:, col_name] = pd.read_pickle(save_path)
